%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Step_generator.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
%
% DESCRIPTION:
%   Step generator
%
% INPUT: 
%
% OUTPUT:  delta, alpha, d_vec, stepsize_vec, trstep, DDf, cDDf, bDDf,mem
%
% USAGE:
% 
% SUBROUTINE CALLS: Step_finder,Scaler, History, Hessian_product,
%                   invHessian_product
%      
%
function [delta, alpha, d_vec, stepsize_vec, trstep, DDf, cDDf, bDDf,mem] = ...
    Step_generator(Step_finder,Scaler, History, Hessian_product, invHessian_product, ...
    iter, dim, mem,maxmem, c, beta, gamma,...
    x_mat, gradf_mat, ngradf_vec, stepsize_vec,tr, trstep, alpha_0, verbose)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% choose search direction and length:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(mem>0)
    S_mat         = zeros(dim,mem);
    Y_mat         = zeros(dim,mem);
    [S_mat,Y_mat] = feval(History,dim,mem+1,x_mat(:,maxmem-mem:maxmem),gradf_mat(:,maxmem-mem:maxmem));
    delta         = feval(Scaler,dim,mem,S_mat,Y_mat,alpha_0);
    alpha         = alpha_0;% /delta;
    d_vec     = ...
        feval(Step_finder,Hessian_product, invHessian_product,dim,mem,gradf_mat(:,maxmem), ngradf_vec(iter), S_mat, Y_mat,tr ,delta,alpha, verbose);
    stepsize_vec(iter)  = feval('norm',d_vec);
else
    % steepest descent direction
    delta=0;
    alpha=alpha_0;
    % if we get to this option, the step MUST have been accepted and iter
    % updated
    d_vec = -stepsize_vec(iter)*gradf_mat(:,maxmem)/ngradf_vec(iter);
end
if (abs(stepsize_vec(iter)-tr)<1e-11)
    trstep = trstep+1;
end
DDf     = gradf_mat(:,maxmem)'*d_vec;
while (DDf > 0),
    if(verbose)
        disp('The Step generator yields a direction of nondescent. ')
    end
    % shrink memory and try again
    if (mem>2)
        S_mat=S_mat(:,2:mem);
        Y_mat=Y_mat(:,2:mem);
        delta = feval(Scaler,dim,mem-1, S_mat, Y_mat, alpha_0);
        alpha = alpha_0;%/delta;
        d_vec     = ...
            feval(Step_finder,Hessian_product, invHessian_product, dim, mem-1, gradf_mat(:,maxmem), ngradf_vec(iter), S_mat, Y_mat,tr ,delta,alpha);
    else
        d_vec=-gamma*stepsize_vec(iter)*gradf_mat(:,maxmem)/ngradf_vec(iter); % steepest descent
    end
    mem=mem-1;
    DDf     = gradf_mat(:,maxmem)'*d_vec;
end
%-----------------------------------------
% end choose search direction and length:
%-----------------------------------------
cDDf                = c*DDf;
bDDf                = beta*DDf;

% clear variables to avoid memory leaks
clear Step_finder Scaler History Hessian_product invHessian_product
clear iter dim maxmem c beta gamma
clear x_mat gradf_mat ngradf_vec tr
clear S_mat Y_mat

return

