%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    IFFT.m
%                   Russell Luke
%             Department of MathSci
%              University of Delaware  
%                    Oct 3, 2007 
% Inverse Fourier transform processor.  Takes data in the fourier domain
% that has been properly normalized and arranged (i.e. phase shifted to 
% for centering in the physical domain) by the companion routine FFT.
% First we fftshift the data, then we take the negative sign of every other
% mode in each direction (x and y) to shift the phase, and finally we
% multiply by (sqrt(res))^2 to get the correct intensity scaling.

function f=IFFT(F)

[ydim,xdim]=size(F);
res=max(xdim,ydim);
F=feval('fftshift',F);

if (xdim == ydim)
  F(2:2:res,:)=-F(2:2:res,:);
  F(:,2:2:res)=-F(:,2:2:res);
  F=res*F;
  f=feval('ifft2',F);
else
  F(2:2:res)=-F(2:2:res);
  F=sqrt(res)*F;
  f=feval('ifft',F);
end

