from numpy import array, zeros
from numpy.linalg import norm

from samsara import Samsara, BFGS1_product, cdSY_mat, Dogleg_QN, Explicit_TR

from problems.Burke.burke import Burke_func


def driver():
    Objective = Burke_func
    x0_vec = zeros(7)

    options = {}
    options['verbose'] = True
    options['alpha_0'] = 5e-8
    options['gamma'] = .5
    options['c'] = .01
    options['beta'] = .9999999
    options['eta1'] = .995
    options['eta2'] = .8
    options['eta3'] = .25
    options['maxmem'] = 9
    options['tr'] = 1e+15

    options['ngrad_TOL'] = 1e-12
    options['step_TOL'] = 1e-16
    options['Maxit'] = 1000
    options['orthog_TOL'] = 1e-6

    options['QN_method'] = BFGS1_product
    options['History'] = cdSY_mat
    options['Step_finder'] = Explicit_TR
    options['update_conditions'] = 'Trust Region'
    options['initial_step'] = 1e+5*options['step_TOL']

    s = Samsara(**options)

    # OPTIONS
    ngrad_TOL = 1e-12
    step_TOL = 1e-14
    Maxit = 300

    it = 0
    xold_vec = x0_vec
    fold, gradfold_vec = Objective(xold_vec)

    xnew_vec = array([])
    stepsize = 999.
    fnew = array([])
    gradfnew_vec = array([])
    ngradfnew = 999.

    while it < Maxit and ngradfnew > ngrad_TOL and stepsize > step_TOL:
        xnew_vec, xold_vec, fold, gradfold_vec, stepsize = \
            s.run(xold_vec, xnew_vec, fold, fnew,
                  gradfold_vec, gradfnew_vec)

        # print('=== ' + str(it) + 'th iteration ===')
        # print('xnew_vec =', xnew_vec)
        # print('xold_vec =', xold_vec)
        # print('fold =', fold)
        # print('gradfold_vec =', gradfold_vec)
        # print('stepsize =', stepsize)

        it += 1
        # new funtion evaluation:
        fnew, gradfnew_vec = Objective(xnew_vec)
        # PYTHON PORT ANNOTATION
        # Replaced sqrt(dot(...)) with norm(...)
        ngradfnew = norm(gradfnew_vec)

        # print('fnew =', fnew)
        # print('gradfnew_vec =', gradfnew_vec)
        # print('ngradfnew =', ngradfnew)

    if stepsize <= step_TOL:
        print('Algorithm stagnated:  stepsize tolerance violated.')

    xopt_vec = xnew_vec
    fopt = fnew

    print('xopt_vec =', xopt_vec)
    print('fopt =', fopt)
    return xopt_vec, fopt


if __name__ == '__main__':
    driver()
