from matlab import double
import matlab.engine

from numpy import array, zeros, load, max, abs
from numpy.linalg import norm

import os

from scipy.io import loadmat

from samsara import Samsara

from problems.Burke.burke import Burke_func

eng = matlab.engine.start_matlab()
eng.addpath('../../../matlab')

try:
    os.remove('state.dat')
    os.remove('stephistory.dat')
except Exception:
    pass

EPS = 1e-8


def test_driver():
    Objective = Burke_func
    x0_vec = zeros(7)

    s = Samsara()

    # OPTIONS
    ngrad_TOL = 1e-12
    step_TOL = 1e-14
    Maxit = 3

    it = 0
    xold_vec = x0_vec
    fold, gradfold_vec = Objective(xold_vec)

    xnew_vec = array([])
    stepsize = 999.
    fnew = None
    gradfnew_vec = array([])
    ngradfnew = 999.

    while it < Maxit and ngradfnew > ngrad_TOL and stepsize > step_TOL:
        xnew_m = eng.transpose(double(xnew_vec.tolist()))
        xold_m = eng.transpose(double(xold_vec.tolist()))
        fold_m = fold
        if fnew is None:
            fnew_m = double([])
        else:
            fnew_m = fnew
        gradfold_m = eng.transpose(double(gradfold_vec.tolist()))
        gradfnew_m = eng.transpose(double(gradfnew_vec.tolist()))

        print(50*'#', 'ITERATION', it, 50*'#')
        print(30*'-', 'PYTHON', 30*'-')
        xnew_vec, xold_vec, fold, gradfold_vec, stepsize =\
            s.run(xold_vec, xnew_vec, fold, fnew,
                  gradfold_vec, gradfnew_vec)

        print()
        print(30*'-', 'MATLAB', 30*'-')
        xnew_m, xold_m, fold_m, gradfold_m, stepsize_m =\
            eng.samsara(xold_m, xnew_m, fold_m, fnew_m, gradfold_m, gradfnew_m,
                        nargout=5)

        # read in values from files and print out values/differences
        mat_vars = loadmat('mat_vars.mat')
        py_vars = load('py_vars.npz')
        print()
        print(30*'-', 'VARIABLES', 30*'-')
        for var in py_vars.files:
            py_var = py_vars[var]
            mat_var = mat_vars[var]
            diff = array(mat_var).flatten() - array(py_var).flatten()
            if max(abs(diff)) > EPS:
                print('###', var, 'DIFFERS ###')
                print('MATLAB:')
                print(mat_vars[var])
                print('PYTHON:')
                print(py_vars[var])
                print()
            else:
                print('###', var, 'are the same ###')

        # print('=== ' + str(it) + 'th iteration ===')
        # print('xnew_vec =', xnew_vec)
        # print('xold_vec =', xold_vec)
        # print('fold =', fold)
        # print('gradfold_vec =', gradfold_vec)
        # print('stepsize =', stepsize)

        it += 1
        fnew, gradfnew_vec = Objective(xnew_vec)
        ngradfnew = norm(gradfnew_vec)

        # print('fnew =', fnew)
        # print('gradfnew_vec =', gradfnew_vec)
        # print('ngradfnew =', ngradfnew)

    if stepsize <= step_TOL:
        print('Algorithm stagnated:  stepsize tolerance violated.')

    xopt_vec = xnew_vec
    fopt = fnew

    # py.test should always report back
    assert False
    return xopt_vec, fopt
