"""Tests for the functions from history.py.
"""
import matlab.engine
from numpy import abs, amax, array, random
import os
import sys

# Add samsara to the path.
PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(
    os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from samsara import *

# The upper boundary for the relative error between the MATLAB and
# Python implementations.
EPS = 1e-8
# The number of repetitions for each test.
ITERATIONS = 10

eng = matlab.engine.start_matlab()
eng.addpath('../../matlab')


def gen_params():
    """Generates random parameters for the test functions.
    """
    dim = random.randint(3, 30)
    mem = random.randint(2, dim)
    x_mat = eng.random('unif', -100., 100., dim, dim)
    gradf_mat = eng.random('unif', -100., 100., dim, dim)

    return dim, mem, x_mat, gradf_mat


def test_fdSY_mat():
    for _ in range(ITERATIONS):
        dim, mem, x_mat, gradf_mat = gen_params()
        S_mat, Y_mat = eng.fdSY_mat(dim, mem, x_mat, gradf_mat, nargout=2)
        S_py, Y_py = fdSY_mat(mem, array(x_mat), array(gradf_mat))
        S_diff = array(S_mat).flatten() - S_py.flatten()
        Y_diff = array(Y_mat).flatten() - Y_py.flatten()
        assert amax(abs(S_diff)) < EPS*amax(abs(array(S_mat).flatten()))
        assert amax(abs(Y_diff)) < EPS*amax(abs(array(Y_mat).flatten()))


def test_cdSY_mat():
    for _ in range(ITERATIONS):
        dim, mem, x_mat, gradf_mat = gen_params()
        S_mat, Y_mat = eng.cdSY_mat(dim, mem, x_mat, gradf_mat, nargout=2)
        S_py, Y_py = cdSY_mat(mem, array(x_mat), array(gradf_mat))
        S_diff = array(S_mat).flatten() - S_py.flatten()
        Y_diff = array(Y_mat).flatten() - Y_py.flatten()
        assert amax(abs(S_diff)) < EPS*amax(abs(array(S_mat).flatten()))
        assert amax(abs(Y_diff)) < EPS*amax(abs(array(Y_mat).flatten()))
